<?php
/**
 * Now Playing API
 * Принимает GET-запросы, сохраняет данные и передает их в виджет.
 */

ini_set('log_errors', 1);
ini_set('error_log', __DIR__ . '/php_errors.log');
error_log("API начал выполняться");

// 📌 Очистка кеша перед загрузкой ключа
clearstatcache();

// Пути к файлам
$settings_file = __DIR__ . '/data/settings.json';
$title_file = __DIR__ . '/data/nowplaying_title.txt';
$artwork_file = __DIR__ . '/data/nowplaying_artwork.png';
$history_file = __DIR__ . '/data/nowplaying_history.txt';
$artwork_dir = __DIR__ . '/data/artworks/'; // Папка для миниатюр

// 📌 Читаем API-ключ из настроек
if (file_exists($settings_file)) {
    $settings = json_decode(file_get_contents($settings_file), true);
    $api_key = !empty($settings['api_key']) ? $settings['api_key'] : '111111';
} else {
    $api_key = '111111';
    file_put_contents($settings_file, json_encode(['api_key' => $api_key]));
}

error_log("📌 Текущий API-ключ: " . $api_key);

// Если API вызывается без ключа, возвращаем ключ
if (!isset($_GET['key'])) {
    echo json_encode(['api_key' => $api_key]);
    exit;
}

// Проверка ключа
if ($_GET['key'] !== $api_key) {
    error_log("❌ Ошибка: Неверный ключ доступа!");
    echo json_encode(['error' => 'Invalid key']);
    exit;
}

// Проверка наличия папок
foreach ([$_DIR__ . '/data', $artwork_dir] as $dir) {
    if (!file_exists($dir)) {
        mkdir($dir, 0777, true);
        error_log("📂 Создана папка: $dir");
    }
}

// Проверка наличия файла истории
if (!file_exists($history_file)) {
    if (@file_put_contents($history_file, "") === false) {
        error_log("❌ Ошибка: Не удалось создать nowplaying_history.txt!");
    } else {
        error_log("📄 Создан файл истории треков.");
    }
}

// Чтение текущего трека из файла
$track = file_exists($title_file) ? trim(file_get_contents($title_file)) : 'Нет данных';
error_log("🎵 Текущий трек: " . $track);

// Запись в историю (10 последних треков)
if ($track !== 'Нет данных') {
    $history = file_exists($history_file) ? file($history_file, FILE_IGNORE_NEW_LINES | FILE_SKIP_EMPTY_LINES) : [];
    if (!in_array($track, $history)) {
        array_unshift($history, $track);
        $history = array_slice($history, 0, 11);
        if (@file_put_contents($history_file, implode("\n", $history)) === false) {
            error_log("❌ Ошибка: Не удалось записать историю в nowplaying_history.txt!");
        } else {
            error_log("📜 История обновлена: " . print_r($history, true));
        }
    }
}

// Сохранение уменьшенной копии обложки
if (file_exists($artwork_file)) {
    $thumbnail_path = $artwork_dir . md5($track) . '.jpg';

    if (!file_exists($thumbnail_path)) {
        createThumbnail($artwork_file, $thumbnail_path, 80, 80);
        error_log("🖼 Сохранена миниатюра: $thumbnail_path");
    }
}

// Очистка старых изображений (оставляем только 10 последних)
cleanOldThumbnails($artwork_dir, 10);

// Получаем список миниатюр
$thumbnails = array_values(array_diff(scandir($artwork_dir), ['.', '..']));

// Формируем историю без первой строки (текущего трека)
$filtered_history = array_slice($history, 1);

error_log("📡 Формируем JSON-ответ.");
$data = [
    'track' => $track,
    'artwork' => '/wp-content/plugins/nowplaying-widget/data/nowplaying_artwork.png?' . time(),
    'history' => $filtered_history,
    'thumbnails' => array_map(fn($file) => "/wp-content/plugins/nowplaying-widget/data/artworks/$file", $thumbnails)
];

// Отключаем кэширование
header("Cache-Control: no-cache, no-store, must-revalidate");
header("Pragma: no-cache");
header("Expires: 0");
header("Content-Type: application/json; charset=UTF-8");

// Ответ API
echo json_encode($data, JSON_UNESCAPED_UNICODE | JSON_PRETTY_PRINT);

error_log("✅ JSON отправлен.");
exit;

// Функция создания уменьшенной копии изображения
function createThumbnail($source, $destination, $width, $height) {
    list($originalWidth, $originalHeight) = getimagesize($source);
    $srcImage = imagecreatefrompng($source);
    $thumbImage = imagecreatetruecolor($width, $height);

    imagecopyresampled($thumbImage, $srcImage, 0, 0, 0, 0, $width, $height, $originalWidth, $originalHeight);
    imagejpeg($thumbImage, $destination, 90);
    imagedestroy($srcImage);
    imagedestroy($thumbImage);
}

// Функция очистки старых миниатюр
function cleanOldThumbnails($directory, $maxFiles) {
    $files = array_diff(scandir($directory, SCANDIR_SORT_DESCENDING), ['.', '..']);
    if (count($files) > $maxFiles) {
        $filesToDelete = array_slice($files, $maxFiles);
        foreach ($filesToDelete as $file) {
            unlink($directory . $file);
            error_log("🗑 Удалён устаревший файл: $file");
        }
    }
}
